@@ -12,13 +12,13 @@ extern void reiser4_fs_close(reiser4_fs_t *fs);
 
 extern errno_t reiser4_fs_root_key(reiser4_fs_t *fs, reiser4_key_t *key);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 extern reiser4_fs_t *reiser4_fs_open(aal_device_t *device, bool_t check);
 #else
 extern reiser4_fs_t *reiser4_fs_open(aal_device_t *device);
 #endif
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 
 #define MASTER_PACK_SIGN "MSTR"
 #define STATUS_PACK_SIGN "STAT"
