@@ -6,14 +6,18 @@
 #include <reiser4/libreiser4.h>
 
 /* Opens filesystem on specified device */
-reiser4_fs_t *reiser4_fs_open(aal_device_t *device,
-			      bool_t check)
-{
-#ifndef ENABLE_STAND_ALONE
+
+#ifndef ENABLE_MINIMAL
+reiser4_fs_t *reiser4_fs_open(aal_device_t *device, bool_t check) {
+#else
+reiser4_fs_t *reiser4_fs_open(aal_device_t *device) {
+#endif
+	reiser4_fs_t *fs;
+
+#ifndef ENABLE_MINIMAL
 	count_t blocks;
 	uint32_t blksize;
 #endif
-	reiser4_fs_t *fs;
 
 	aal_assert("umka-148", device != NULL);
 
@@ -27,7 +31,7 @@ reiser4_fs_t *reiser4_fs_open(aal_device_t *device,
 	if (!(fs->master = reiser4_master_open(device)))
 		goto error_free_fs;
     
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	if (check) {
 		if (reiser4_master_valid(fs->master))
 			goto error_free_master;
@@ -43,7 +47,7 @@ reiser4_fs_t *reiser4_fs_open(aal_device_t *device,
 	if (!(fs->format = reiser4_format_open(fs)))
 		goto error_free_status;
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	if (check) {
 		if (reiser4_format_valid(fs->format))
 			goto error_free_format;
@@ -58,7 +62,8 @@ reiser4_fs_t *reiser4_fs_open(aal_device_t *device,
 
 	if (check) {
 		if (reiser4_alloc_valid(fs->alloc)) {
-			aal_warn("Block allocator data seems corrupted.");
+			aal_error("Block allocator data seems corrupted.");
+			goto error_free_alloc;
 		}
 	}
 	
@@ -70,26 +75,33 @@ reiser4_fs_t *reiser4_fs_open(aal_device_t *device,
 		if (reiser4_oid_valid(fs->oid))
 			goto error_free_oid;
 	}
-
-	if (reiser4_fs_init_params(fs))
-		goto error_free_oid;
 #endif
-	
 	if (!(fs->tree = reiser4_tree_init(fs)))
 		goto error_free_oid;
+
+
+#ifndef ENABLE_MINIMAL
+	if (check) {
+		if (reiser4_opset_init(fs->tree, check))
+			goto error_free_oid;
+	}
+#else
+	if (reiser4_opset_init(fs->tree))
+		goto error_free_oid;
+#endif
 	
 	return fs;
 
  error_free_oid:
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	reiser4_oid_close(fs->oid);
  error_free_alloc:
 	reiser4_alloc_close(fs->alloc);
  error_free_format:
-	reiser4_format_close(fs->format);
 #endif
+	reiser4_format_close(fs->format);
  error_free_status:
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	reiser4_status_close(fs->status);
  error_free_master:
 #endif
@@ -104,13 +116,15 @@ void reiser4_fs_close(reiser4_fs_t *fs) {
     
 	aal_assert("umka-230", fs != NULL);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	if (!aal_device_readonly(fs->device))
 		reiser4_fs_sync(fs);
-#endif
 	reiser4_tree_fini(fs->tree);
-	
-#ifndef ENABLE_STAND_ALONE
+#else
+	reiser4_tree_close(fs->tree);
+#endif
+
+#ifndef ENABLE_MINIMAL
 	reiser4_oid_close(fs->oid);
 	reiser4_alloc_close(fs->alloc);
 #endif
@@ -118,7 +132,7 @@ void reiser4_fs_close(reiser4_fs_t *fs) {
 	reiser4_format_close(fs->format);
 	reiser4_master_close(fs->master);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	reiser4_status_close(fs->status);
 #endif
 	
@@ -126,9 +140,9 @@ void reiser4_fs_close(reiser4_fs_t *fs) {
 	aal_free(fs);
 }
 
-#ifndef ENABLE_STAND_ALONE
-static errno_t callback_check_block(void *entity, blk_t start,
-				    count_t width, void *data)
+#ifndef ENABLE_MINIMAL
+static errno_t cb_check_block(void *entity, blk_t start,
+			      count_t width, void *data)
 {
 	blk_t blk = *(blk_t *)data;
 	return (blk >= start && blk < start + width);
@@ -142,36 +156,33 @@ reiser4_owner_t reiser4_fs_belongs(
 	aal_assert("umka-1534", fs != NULL);
 
 	/* Checks if passed @blk is master super block */
-	if (reiser4_master_layout(fs->master, callback_check_block, &blk))
+	if (reiser4_master_layout(fs->master, cb_check_block, &blk))
 		return O_MASTER;
 	
 	/* Checks if passed @blk belongs to format metadata */
-	if (reiser4_format_layout(fs->format, callback_check_block, &blk))
+	if (reiser4_format_layout(fs->format, cb_check_block, &blk))
 		return O_FORMAT;
 
 	/* Checks if passed @blk belongs to oid allocator metadata */
-	if (reiser4_oid_layout(fs->oid, callback_check_block, &blk))
+	if (reiser4_oid_layout(fs->oid, cb_check_block, &blk))
 		return O_OID;
 
 	/* Checks if passed @blk belongs to journal metadata if journal
 	   opened. */
 	if (fs->journal) {
-		if (reiser4_journal_layout(fs->journal,
-					   callback_check_block, &blk))
-		{
+		if (reiser4_journal_layout(fs->journal, cb_check_block, &blk))
 			return O_JOURNAL;
-		}
 	}
 
 	/* Check if @blk is filesystem status block. */
-	if (reiser4_status_layout(fs->status, callback_check_block, &blk))
+	if (reiser4_status_layout(fs->status, cb_check_block, &blk))
 		return O_STATUS;
 	
 	/* Checks if passed @blk belongs to block allocator data */
-	if (reiser4_alloc_layout(fs->alloc, callback_check_block, &blk))
+	if (reiser4_alloc_layout(fs->alloc, cb_check_block, &blk))
 		return O_ALLOC;
 
-	if (reiser4_backup_layout(fs, callback_check_block, &blk))
+	if (reiser4_backup_layout(fs, cb_check_block, &blk))
 		return O_BACKUP;
 	
 	return O_UNKNOWN;
@@ -212,8 +223,8 @@ errno_t reiser4_fs_layout(reiser4_fs_t *fs,
 	return reiser4_backup_layout(fs, region_func, data);
 }
 
-static errno_t callback_mark_block(void *entity, blk_t start,
-				   count_t width, void *data)
+static errno_t cb_mark_block(void *entity, blk_t start,
+			     count_t width, void *data)
 {
 	return reiser4_alloc_occupy((reiser4_alloc_t *)data,
 				    start, width);
@@ -251,8 +262,7 @@ reiser4_fs_t *reiser4_fs_create(
 	aal_device_t *device,           /* device filesystem will be lie on */
 	fs_hint_t *hint)                /* filesystem hint */
 {
-	rid_t policy;
-	rid_t format;
+	reiser4_plug_t *format, *policy, *key;
 
 	count_t free;
 	reiser4_fs_t *fs;
@@ -274,7 +284,7 @@ reiser4_fs_t *reiser4_fs_create(
 	fs->device = device;
 		
 	/* Create master super block. */
-	format = reiser4_param_value("format");
+	format = reiser4_profile_plug(PROF_FORMAT);
 		
 	if (!(fs->master = reiser4_master_create(device, hint->blksize)))
 		goto error_free_fs;
@@ -283,7 +293,7 @@ reiser4_fs_t *reiser4_fs_create(
 		goto error_free_master;
 
 	/* Setting up master super block. */
-	reiser4_master_set_format(fs->master, format);
+	reiser4_master_set_format(fs->master, format->id.id);
 	reiser4_master_set_uuid(fs->master, hint->uuid);
 	reiser4_master_set_label(fs->master, hint->label);
 
@@ -291,22 +301,24 @@ reiser4_fs_t *reiser4_fs_create(
 		goto error_free_master;
 
 	/* Getting tail policy from default params. */
-	policy = reiser4_param_value("policy");
+	policy = reiser4_profile_plug(PROF_POLICY);
 	
 	/* Creates disk format. */
-	if (!(fs->format = reiser4_format_create(fs, hint->blocks,
-						 policy, format)))
+	if (!(fs->format = reiser4_format_create(fs, format, policy, 
+						 hint->blocks)))
 	{
 		goto error_free_status;
 	}
 
 	/* Taking care about key flags in format super block */
-	if (reiser4_param_value("key") == KEY_LARGE_ID) {
-		plug_call(fs->format->entity->plug->o.format_ops, set_flags,
-			  fs->format->entity, (1 << REISER4_LARGE_KEYS));
+	key = reiser4_profile_plug(PROF_KEY);
+	
+	if (key->id.id == KEY_LARGE_ID) {
+		plug_call(fs->format->ent->plug->o.format_ops, set_flags,
+			  fs->format->ent, (1 << REISER4_LARGE_KEYS));
 	} else {
-		plug_call(fs->format->entity->plug->o.format_ops,
-			  set_flags, fs->format->entity, 0);
+		plug_call(fs->format->ent->plug->o.format_ops,
+			  set_flags, fs->format->ent, 0);
 	}
 
 	/* Creates block allocator */
@@ -323,7 +335,7 @@ reiser4_fs_t *reiser4_fs_create(
 	if (!(fs->backup = reiser4_backup_create(fs)))
 		goto error_free_tree;
 	
-	if (reiser4_fs_layout(fs, callback_mark_block, fs->alloc)) {
+	if (reiser4_fs_layout(fs, cb_mark_block, fs->alloc)) {
 		aal_error("Can't mark filesystem blocks used.");
 		goto error_free_backup;
 	}
@@ -414,40 +426,6 @@ errno_t reiser4_fs_sync(
 	return reiser4_status_sync(fs->status);
 }
 
-/* Set default fs params into the profile unless they are overridden 
-   (like a user specified they by hands). */
-errno_t reiser4_fs_init_params(reiser4_fs_t *fs) {
-	uint16_t flags, pid;
-	
-	if (!reiser4_param_get_flag("format", PF_OVERRIDDEN)) {
-		pid = reiser4_master_get_format(fs->master);
-		reiser4_param_set("format", pid);
-		reiser4_param_set_flag("format", PF_READ);
-	}
-	
-	if (!reiser4_param_get_flag("key", PF_OVERRIDDEN)) {
-		flags = plug_call(fs->format->entity->plug->o.format_ops,
-				  get_flags, fs->format->entity);
-
-		/* Set the default fs key plugin in the profile. */
-		if ((1 << REISER4_LARGE_KEYS) & flags) {
-			reiser4_param_override("key", "key_large");
-		} else {
-			reiser4_param_override("key", "key_short");
-		}
-
-		reiser4_param_set_flag("key", PF_READ);
-	}
-
-	if (!reiser4_param_get_flag("policy", PF_OVERRIDDEN)) {
-		pid = reiser4_format_get_policy(fs->format);
-		reiser4_param_set("policy", pid);
-		reiser4_param_set_flag("policy", PF_READ);
-	}
-	
-	return 0;
-}
-
 #endif
 
 /* Returns the key of the fake root parent */
@@ -456,26 +434,19 @@ errno_t reiser4_fs_root_key(reiser4_fs_t *fs,
 {
 	oid_t locality;
 	oid_t objectid;
-	rid_t pid;
 	
 	aal_assert("umka-1949", fs != NULL);
 	aal_assert("umka-1950", key != NULL);
 
-	pid = reiser4_format_key_pid(fs->format);
-
-	/* Finding needed key plugin by its identifier. */
-	if (!(key->plug = reiser4_factory_ifind(KEY_PLUG_TYPE, pid))) {
-		aal_error("Can't find key plugin by its id 0x%x.", pid);
-		return -EINVAL;
-	}
+	key->plug = fs->tree->ent.tpset[TPSET_KEY];
 	
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	locality = reiser4_oid_root_locality(fs->oid);
 	objectid = reiser4_oid_root_objectid(fs->oid);
 #else
 	locality = REISER4_ROOT_LOCALITY;
 	objectid = REISER4_ROOT_OBJECTID;
 #endif
-	return reiser4_key_build_generic(key, KEY_STATDATA_TYPE,
-					 locality, 0, objectid, 0);
+	return plug_call(key->plug->o.key_ops, build_generic, key,
+			 KEY_STATDATA_TYPE, locality, 0, objectid, 0);
 }
