@@ -24,7 +24,7 @@ static bool_t reiser4_tree_root_node(reiser4_tree_t *tree,
 	return reiser4_tree_get_root(tree) == node->block->nr;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Updates root block number in format by passed @blk. Takes care about correct
    block number in loaded root node if any. */
 void reiser4_tree_set_root(reiser4_tree_t *tree, blk_t blk) {
@@ -86,7 +86,7 @@ blk_t reiser4_tree_get_root(reiser4_tree_t *tree) {
 			 get_root, tree->fs->format->ent);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Returns tree height stored in format. */
 uint8_t reiser4_tree_get_height(reiser4_tree_t *tree) {
 	aal_assert("umka-2411", tree != NULL);
@@ -162,7 +162,7 @@ static errno_t reiser4_tree_unhash_node(reiser4_tree_t *tree,
 	return aal_hash_table_remove(tree->nodes, &blk);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Acknowledles, that passed @place has nodeptr that points onto passed
    @node. This is needed for tree_rebind_node() function. */
 static int tree_check_pos(reiser4_place_t *place, blk_t blocknr) {
@@ -199,7 +199,7 @@ static errno_t tree_find_child_pos(reiser4_tree_t *tree,
 	lookup_hint_t hint;
         reiser4_key_t lkey;
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	uint32_t i;
 #endif
     
@@ -209,7 +209,7 @@ static errno_t tree_find_child_pos(reiser4_tree_t *tree,
 
 	place->node = parent;
 	
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Checking if we are in position already. */
 	if (tree_check_pos(place, child->block->nr))
 		goto out_correct_place;
@@ -222,14 +222,14 @@ static errno_t tree_find_child_pos(reiser4_tree_t *tree,
         if (reiser4_node_lookup(parent, &hint, FIND_EXACT,
 				&place->pos) == PRESENT)
 	{
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 		if (tree_check_pos(place, child->block->nr))
 			goto out_correct_place;
 #endif
 	}
 
 	/* Getting position by means of linear traverse. */
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	for (i = 0; i < reiser4_node_items(place->node); i++) {
 		uint32_t j;
 		errno_t res;
@@ -290,13 +290,6 @@ errno_t reiser4_tree_load_root(reiser4_tree_t *tree) {
 	if (tree->root)
 		return 0;
 
-#ifndef ENABLE_STAND_ALONE
-	/* Check if tree contains some nodes at all. It does not contain them
-	   just after creation. The is root blk in format is set to INVAL_BLK.*/
-	if (reiser4_tree_fresh(tree))
-		return -EINVAL;
-#endif
-
 	/* Getting root node and loading it. */
 	root_blk = reiser4_tree_get_root(tree);
 	
@@ -352,7 +345,7 @@ static errno_t debug_node_check_keys(reiser4_node_t *node) {
 
 #endif
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Assignes passed @node to root. Takes care about root block number and tree
    height in format. */
 errno_t reiser4_tree_assign_root(reiser4_tree_t *tree,
@@ -475,7 +468,7 @@ errno_t reiser4_tree_disconnect_node(reiser4_tree_t *tree,
 	return 0;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Updates all internal node loaded children positions in parent. */
 static errno_t reiser4_tree_update_node(reiser4_tree_t *tree,
 					reiser4_node_t *node)
@@ -581,7 +574,7 @@ errno_t reiser4_tree_unload_node(reiser4_tree_t *tree, reiser4_node_t *node) {
 	aal_assert("umka-1840", tree != NULL);
 	aal_assert("umka-1842", node != NULL);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Check if node is dirty. */
 	if (reiser4_node_isdirty(node)) {
 		aal_warn("Unloading dirty node %llu.",
@@ -653,7 +646,7 @@ static int reiser4_tree_neig_place(reiser4_tree_t *tree,
 		return 0;
 	
 	/* Position correcting. We do not use place_inc() and place_dec() here,
-	   because they are not accessible in stand alone mode and we do not
+	   because they are not accessible in minimal mode and we do not
 	   want to make it accessible because here is one place only and they
 	   are quite big. */
 	place->pos.item += (where == DIR_LEFT ? -1 : 1);
@@ -823,7 +816,7 @@ errno_t reiser4_tree_place_key(reiser4_tree_t *tree,
 	return 0;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Gets the key of the next item. */
 static errno_t reiser4_tree_next_key(reiser4_tree_t *tree, 
 				     reiser4_place_t *place, 
@@ -995,7 +988,11 @@ static int cb_nodes_comp_func(void *key1, void *key2, void *data) {
 	return 0;
 }
 
-#define TREE_NODES_TABLE_SIZE (512)
+#ifndef ENABLE_MINIMAL
+# define TREE_NODES_TABLE_SIZE (512)
+#else
+# define TREE_NODES_TABLE_SIZE (32)
+#endif
 #define TREE_BLOCKS_TABLE_SIZE (512)
 
 /* Initializes tree instance on passed filesystem and return it to caller. Then
@@ -1023,7 +1020,7 @@ reiser4_tree_t *reiser4_tree_init(reiser4_fs_t *fs) {
 		goto error_free_tree;
 	}
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Initializing hash table for storing loaded unformatted blocks in
 	   it. This uses all callbacks we described above for getting hash
 	   values, lookup, etc. */
@@ -1050,7 +1047,7 @@ reiser4_tree_t *reiser4_tree_init(reiser4_fs_t *fs) {
 	return tree;
 
  error_free_data:
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	aal_hash_table_free(tree->blocks);
 error_free_nodes:
 #endif
@@ -1060,7 +1057,7 @@ error_free_nodes:
 	return NULL;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Closes specified tree. */
 void reiser4_tree_fini(reiser4_tree_t *tree) {
 	aal_assert("umka-134", tree != NULL);
@@ -1082,7 +1079,7 @@ errno_t reiser4_tree_collapse(reiser4_tree_t *tree) {
                 return 0;
                                                                                           
         return reiser4_tree_walk_node(tree, tree->root, 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 				      NULL, NULL,
 #endif
                                       reiser4_tree_unload_node);
@@ -1090,7 +1087,7 @@ errno_t reiser4_tree_collapse(reiser4_tree_t *tree) {
 
 /* Closes specified tree without saving dirty nodes to device. It just thows out
    all loaded nodes without dealing with allocating etc. This may be used in
-   stand alone mode and/or just to free modified tree without a changes on
+   minimal mode and/or just to free modified tree without a changes on
    device. */
 void reiser4_tree_close(reiser4_tree_t *tree) {
 	aal_assert("vpf-1316", tree != NULL);
@@ -1099,7 +1096,7 @@ void reiser4_tree_close(reiser4_tree_t *tree) {
 	reiser4_tree_collapse(tree);
 
 	/* Releasing unformatted nodes hash table. */
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	aal_hash_table_free(tree->blocks);
 #endif
 
@@ -1110,7 +1107,7 @@ void reiser4_tree_close(reiser4_tree_t *tree) {
 	tree->fs->tree = NULL;
 	aal_free(tree);
 }
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 static errno_t cb_flags_dup(reiser4_place_t *place, void *data) {
 	reiser4_item_dup_flags(place, *(uint16_t *)data);
 	return 0;
@@ -1303,7 +1300,7 @@ static errno_t cb_nodeptr_adjust(reiser4_tree_t *tree, reiser4_place_t *place) {
 #endif
 
 static errno_t cb_node_unload(reiser4_tree_t *tree, reiser4_node_t *node) {
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	count_t free_blocks;
 
 	/* Updating free space counter in format for the case some blocks were 
@@ -1318,7 +1315,7 @@ static errno_t cb_node_unload(reiser4_tree_t *tree, reiser4_node_t *node) {
 	if (reiser4_node_locked(node))
 		return 0;
 	
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Okay, node is fully allocated now and ready to be saved to device if
 	   it is dirty. */
 	if (reiser4_node_isdirty(node) && reiser4_node_sync(node)) {
@@ -1343,7 +1340,7 @@ errno_t reiser4_tree_adjust(reiser4_tree_t *tree) {
 		/* Check for special case -- tree_adjust() is calling during
 		   tree_growup(), when empty root is connected. */
 		if (reiser4_node_items(tree->root))
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 			res = reiser4_tree_walk_node(tree, tree->root, 
 						     cb_node_adjust,
 						     cb_nodeptr_adjust,
@@ -1364,7 +1361,7 @@ errno_t reiser4_tree_adjust(reiser4_tree_t *tree) {
 /* Walking though the tree cache and closing all nodes. */
 errno_t reiser4_tree_walk_node(reiser4_tree_t *tree, 
 			       reiser4_node_t *node,
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 			       walk_func_t pre_func, 
 			       walk_on_func_t on_func,
 #endif
@@ -1376,7 +1373,7 @@ errno_t reiser4_tree_walk_node(reiser4_tree_t *tree,
 	aal_assert("umka-1933", tree != NULL);
 	aal_assert("umka-1934", node != NULL);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	if (pre_func && (res = pre_func(tree, node)))
 		return res;
 #endif
@@ -1394,7 +1391,7 @@ errno_t reiser4_tree_walk_node(reiser4_tree_t *tree,
 		if ((res = reiser4_place_fetch(&place)))
 			return res;
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 		if (on_func && (res = on_func(tree, &place)))
 			return res;
 #endif
@@ -1416,7 +1413,7 @@ errno_t reiser4_tree_walk_node(reiser4_tree_t *tree,
 
 			/* Making recursive call to tree_walk_node(). */
 			if ((res = reiser4_tree_walk_node(tree, child, 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 							  pre_func, on_func, 
 #endif
 							  post_func)))
@@ -1432,7 +1429,7 @@ errno_t reiser4_tree_walk_node(reiser4_tree_t *tree,
 	return post_func ? post_func(tree, node) : 0;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Helper function for save one unformatted block to device. Used from
    tree_sync() to save all in-memory unfromatted blocks. */
 static errno_t cb_save_block( void *entry, void *data) {
@@ -1778,7 +1775,7 @@ lookup_t reiser4_tree_lookup(reiser4_tree_t *tree, lookup_hint_t *hint,
 	   having some garbage in it. */
 	aal_memset(place, 0, sizeof(*place));
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Making sure that root exists. If not, getting out with @place
 	   initialized by NULL root. */
 	if (reiser4_tree_fresh(tree)) {
@@ -1790,7 +1787,7 @@ lookup_t reiser4_tree_lookup(reiser4_tree_t *tree, lookup_hint_t *hint,
 			return res;
 		
 		reiser4_place_assign(place, tree->root, 0, MAX_UINT32);
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	}
 #endif
 
@@ -1810,7 +1807,7 @@ lookup_t reiser4_tree_lookup(reiser4_tree_t *tree, lookup_hint_t *hint,
 		   or some error occured during last node lookup. */
 		if (clevel <= hint->level || res < 0) {
 			if (res == PRESENT) {
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 				if (reiser4_tree_collision_start(tree, place,
 								 &wanted))
 				{
@@ -1863,7 +1860,7 @@ lookup_t reiser4_tree_lookup(reiser4_tree_t *tree, lookup_hint_t *hint,
 	restore_and_exit(ABSENT);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Returns 1 if passed @tree has minimal possible height and thus cannot be
    dried out. Othersize 0 is returned. */
 bool_t reiser4_tree_minimal(reiser4_tree_t *tree) {
@@ -2287,18 +2284,20 @@ static errno_t tree_shift_todir(reiser4_tree_t *tree, reiser4_place_t *place,
 }
 
 /* This calculates if space in passed @needed is enough for passed @needed. */
-static inline int32_t tree_calc_space(reiser4_place_t *place) {
-	uint32_t overhead = reiser4_node_overhead(place->node);
+static inline int32_t tree_calc_space(reiser4_place_t *place, 
+				      uint32_t ioverh) 
+{
+	uint32_t overh = reiser4_node_overhead(place->node);
 	
 	return reiser4_node_space(place->node) - 
-		(place->pos.unit == MAX_UINT32 ? overhead : 0);
+		(place->pos.unit == MAX_UINT32 ? (overh + ioverh) : 0);
 }
 
-/* Makes space in tree to insert @needed bytes of data. Returns space in insert
+/* Makes space in tree to insert @ilen bytes of data. Returns space in insert
    point, or negative value for errors. */
 int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
-			    reiser4_place_t *parent, uint32_t needed, 
-			    uint32_t flags)
+			    reiser4_place_t *parent, uint32_t ilen, 
+			    uint32_t ioverh, uint32_t flags)
 {
 	int alloc;
 	errno_t res;
@@ -2309,7 +2308,8 @@ int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
 	aal_assert("umka-766", place != NULL);
 	
 	aal_assert("umka-3064", place->node != NULL);
-	aal_assert("vpf-1543", needed <= reiser4_node_maxspace(place->node));
+	aal_assert("vpf-1543", ilen + ioverh <= 
+		   reiser4_node_maxspace(place->node));
 
 	if (reiser4_tree_fresh(tree))
 		return -EINVAL;
@@ -2317,7 +2317,7 @@ int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
 	/* Check if there is enough space in insert point node. If so -- do
 	   nothing but exit. Here is also check if node is empty. Then we exit
 	   too and return available space in it. */
-	if ((enough = tree_calc_space(place)) >= (int32_t)needed)
+	if ((enough = tree_calc_space(place, ioverh)) >= (int32_t)ilen)
 		return enough;
 
 	/* Shifting data into left neighbour if it exists and left shift
@@ -2328,7 +2328,7 @@ int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
 		if ((res = tree_shift_todir(tree, place, flags, DIR_LEFT)))
 			return res;
 		
-		if ((enough = tree_calc_space(place)) >= (int32_t)needed)
+		if ((enough = tree_calc_space(place, ioverh)) >= (int32_t)ilen)
 			return enough;
 	}
 
@@ -2340,23 +2340,24 @@ int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
 		if ((res = tree_shift_todir(tree, place, flags, DIR_RIGHT)))
 			return res;
 		
-		if ((enough = tree_calc_space(place)) >= (int32_t)needed)
+		if ((enough = tree_calc_space(place, ioverh)) >= (int32_t)ilen)
 			return enough;
 	}
 
 	/* Check if we allowed to allocate new nodes if there still not enough
-	   of space for insert @needed bytes of data to tree. */
+	   of space for insert @ilen bytes of data to tree. */
 	if (!(SF_ALLOW_ALLOC & flags))
-		return tree_calc_space(place);
+		return tree_calc_space(place, ioverh);
 	
 	/* Here we still have not enough free space for inserting item/unit into
 	   the tree. Allocating new nodes and trying to shift data into
 	   them. There are possible two tries to allocate new node and shift
 	   insert point to one of them. */
-	for (alloc = 0; enough < (int32_t)needed && alloc < 2; alloc++) {
+	for (alloc = 0; enough < (int32_t)ilen && alloc < 2; alloc++) {
 		reiser4_place_t save;
 		reiser4_node_t *node;
 		uint32_t shift_flags;
+		uint32_t left_items;
 		reiser4_place_t aplace;
 
 		/* Saving place as it will be usefull for us later */
@@ -2403,7 +2404,9 @@ int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
 			tree_next_child_pos(save.node, &aplace);
 		}
 		
-		if (reiser4_node_items(save.node) == 0) {
+		left_items = reiser4_node_items(save.node);
+		
+		if (left_items == 0) {
 			reiser4_node_lock(save.node);
 
 			/* If evth was moved to the new allocated node, it will
@@ -2459,15 +2462,20 @@ int32_t reiser4_tree_expand(reiser4_tree_t *tree, reiser4_place_t *place,
 			aal_memcpy(parent, &aplace, sizeof(aplace));
 		}
 		
+		/* If we are still in the same node, but on not-esistent 
+		   anymore item, set unit position to MAX_UINT32. */
+		if (save.node == place->node && place->pos.item >= left_items)
+			place->pos.unit = MAX_UINT32;
+	
 		/* Checking if it is enough of space in @place. */
-		enough = tree_calc_space(place);
+		enough = tree_calc_space(place, ioverh);
 
 		/* If it is not enough of space and insert point was actually
 		   moved to neighbour node, we set @place to @save and give it
 		   yet another try to make space. */
-		if (enough < (int32_t)needed && save.node != place->node) {
+		if (enough < (int32_t)ilen && save.node != place->node) {
 			aal_memcpy(place, &save, sizeof(save));
-			enough = tree_calc_space(place);
+			enough = tree_calc_space(place, ioverh);
 		}
 
 		/* Releasing new allocated @node if it is empty and insert point
@@ -2790,7 +2798,6 @@ int64_t reiser4_tree_modify(reiser4_tree_t *tree, reiser4_place_t *place,
 	errno_t res;
 	int32_t space;
 	int32_t write;
-	uint32_t needed;
 	reiser4_place_t aplace;
 
 	aal_assert("umka-2673", tree != NULL);
@@ -2838,28 +2845,28 @@ int64_t reiser4_tree_modify(reiser4_tree_t *tree, reiser4_place_t *place,
 		return res;
 	
 	/* Needed space to be prepared in tree. */
-	needed = hint->len + hint->overhead;
 	mode = (place->pos.unit == MAX_UINT32);
-
+		
 	/* Preparing space in tree. */
-	space = reiser4_tree_expand(tree, place, &aplace,
-				    needed, hint->shift_flags);
+	space = reiser4_tree_expand(tree, place, &aplace, hint->len, 
+				    hint->overhead, hint->shift_flags);
 	
+
+	/* Needed space is the length of the item + an item overhead on 
+	   the item creation if needed. */
+	if (place->pos.unit != MAX_UINT32)
+		hint->overhead = 0;
+
 	if (space < 0) {
 		aal_error("Can't prepare space in tree.");
 		return space;
 	} else {
 		/* Checking if we still have less space than needed. This is
 		   ENOSPC case if we tried to insert data. */
-		if ((uint32_t)space < needed)
+		if (space < hint->len)
 			return -ENOSPC;
 	}
 
-	/* Initializing @hint->overhead if insert mode get changed after making
-	   space in the tree. */
-	if (mode != (place->pos.unit == MAX_UINT32))
-		hint->overhead = reiser4_item_overhead(hint->plug);
-
 	/* Inserting/writing data to node. */
 	if ((write = reiser4_node_modify(place->node, &place->pos,
 					 hint, modify_func)) < 0)
@@ -3159,7 +3166,6 @@ errno_t reiser4_tree_scan(reiser4_tree_t *tree,
 			  void *data) 
 {
         reiser4_key_t key, max;
-        uint32_t count;
         errno_t res;
 
         aal_assert("vpf-1423", tree != NULL);
@@ -3186,7 +3192,8 @@ errno_t reiser4_tree_scan(reiser4_tree_t *tree,
                 lookup_t lookup;
 		pos_t *pos;
 
-                /* FIXME-VITALY: This is not key-collision-safe. */
+                /* Some items can be handled a few times due to key 
+		   collisions. */
 		hint.key = &key;
 		hint.level = LEAF_LEVEL;
 		hint.collision = NULL;
@@ -3206,7 +3213,25 @@ errno_t reiser4_tree_scan(reiser4_tree_t *tree,
 			if (res) continue;
 		}
 		
-                for (; pos->item < reiser4_node_items(place.node); pos->item++)	{
+		/* Count may get change in func (e.g. after item fusing), so 
+		   get it on every loop */
+		while (1) {
+			if (pos->item >= reiser4_node_items(place.node)) {
+				/* All items are handled whithin this node, but 
+				   lookup is not needed. To avoid infinite loop 
+				   in the case of key collision, get the next item 
+				   instead of another lookup call. */
+				if ((res = reiser4_tree_next_place(tree, &place,
+								   &place))) 
+				{
+					aal_error("Failed to get the next node.");
+					return res;
+				}
+
+				if (!place.node)
+					return 0;
+			}
+				
                         if ((res = reiser4_place_fetch(&place)))
                                 return res;
 			
@@ -3229,9 +3254,7 @@ errno_t reiser4_tree_scan(reiser4_tree_t *tree,
 					if (res) break;
 				}
 
-				count = reiser4_node_items(place.node);
 				place.pos.item = -1;
-				
 				continue;
 			}
 			
@@ -3245,6 +3268,8 @@ errno_t reiser4_tree_scan(reiser4_tree_t *tree,
 
                         /* If res != 0, lookup is needed. */
                         if (res) break;
+
+			pos->item++;
                 }
         }
 
