@@ -5,15 +5,23 @@
 
 #include <repair/semantic.h>
 
-static void repair_semantic_lost_name(reiser4_object_t *object, char *name) {
+static void repair_semantic_lost_name(reiser4_object_t *object, 
+				      char *name, int namelen) 
+{
+	uint8_t len1, len2;
 	char *key;
-	uint8_t len;
 
-	len = aal_strlen(LOST_PREFIX);
-	key = reiser4_print_key(&object->ent->object, PO_INODE);
+	len1 = aal_strlen(LOST_PREFIX);
+	key = reiser4_print_inode(&object->ent->object);
 	
-	aal_memcpy(name, LOST_PREFIX, len);
-	aal_memcpy(name + len, key, aal_strlen(key));
+	len2 = aal_strlen(key);
+	
+	if (len1 + len2 >= namelen)
+		len2 = namelen - len1 - 1;
+		
+	aal_memcpy(name, LOST_PREFIX, len1);
+	aal_memcpy(name + len1, key, len2);
+	name[len1 + len2] = 0;
 }
 
 /* Callback for repair_object_check_struct. Mark the passed item as CHECKED. */
@@ -24,7 +32,7 @@ static errno_t cb_register_item(reiser4_place_t *place, void *data) {
                 fsck_mess("Node (%llu), item (%u), [%s]: item registering "
 			  "failed, it belongs to another object already.",
 			  place_blknr(place), place->pos.item,
-			  reiser4_print_key(&place->key, PO_DEFAULT));
+			  reiser4_print_key(&place->key));
                 return -EINVAL;
         }
          
@@ -129,37 +137,11 @@ static errno_t repair_semantic_add_entry(reiser4_object_t *parent,
 	
 	if ((res = reiser4_object_add_entry(parent, &entry)))
 		aal_error("Can't add entry %s to %s.", name, 
-			  reiser4_print_key(&parent->ent->object, PO_INODE));
+			  reiser4_print_inode(&parent->ent->object));
 	
 	return res;
 }
 
-static errno_t repair_semantic_unlink(repair_semantic_t *sem, 
-				      reiser4_object_t *parent,
-				      reiser4_object_t *object,
-				      char *name)
-{
-	entry_hint_t entry;
-	
-	aal_assert("vpf-1336", sem != NULL);
-	aal_assert("vpf-1337", object != NULL);
-
-	if (!parent) {
-		reiser4_plug_t *plug = object->ent->opset.plug[OPSET_OBJ];
-		
-		/* If there is no parent, just detach the object. */
-		if (!plug->o.object_ops->detach) 
-			return 0;
-
-		return plug_call(plug->o.object_ops, detach, 
-				 object->ent, NULL);
-	}
-	
-	/* unlink from the parent. */
-	aal_strncpy(entry.name, name, sizeof(entry.name));
-	return reiser4_object_unlink(parent, &entry);
-}
-
 static errno_t repair_semantic_link_lost(repair_semantic_t *sem,
 					 reiser4_object_t *parent,
 					 reiser4_object_t *object)
@@ -172,10 +154,10 @@ static errno_t repair_semantic_link_lost(repair_semantic_t *sem,
 	aal_assert("vpf-1180", object != NULL);
 	
 	/* Make the lost name. */
-	repair_semantic_lost_name(object, buff);
+	repair_semantic_lost_name(object, buff, REISER4_MAX_BLKSIZE);
 	
 	/* Detach if possible. */
-	if ((res = repair_semantic_unlink(sem, NULL, object, NULL)))
+	if ((res = reiser4_object_detach(object, NULL)))
 		return res;
 	
 	/* Add the entry to the @parent. */
@@ -271,7 +253,7 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 		char buff[REISER4_MAX_BLKSIZE];
 		
 		/* EOF was reached. Add entry to the parent. */
-		repair_semantic_lost_name(object, buff);
+		repair_semantic_lost_name(object, buff, REISER4_MAX_BLKSIZE);
 		if ((res = repair_semantic_add_entry(parent, object, buff)))
 			goto error_parent_close;
 
@@ -301,7 +283,7 @@ static reiser4_object_t *repair_semantic_uplink(repair_semantic_t *sem,
 	
  error_object_detach:
 	/* Detach if possible. */
-	res = repair_semantic_unlink(sem, NULL, object, NULL);
+	res = reiser4_object_detach(object, NULL);
 	return res < 0 ? INVAL_PTR : NULL;
 	
  error_parent_close:
@@ -340,8 +322,8 @@ static reiser4_object_t *cb_object_traverse(reiser4_object_t *parent,
 	if (object == NULL) {
 		fsck_mess("Directory [%s]: can't find the object "
 			  "[%s] pointed by the entry [%s].%s",
-			  reiser4_print_key(&parent->ent->object, PO_INODE),
-			  reiser4_print_key(&entry->object, PO_INODE),
+			  reiser4_print_inode(&parent->ent->object),
+			  reiser4_print_inode(&entry->object),
 			  entry->name, sem->repair->mode != RM_CHECK ? 
 			  " Entry is removed." : "");
 		
@@ -400,11 +382,14 @@ static reiser4_object_t *cb_object_traverse(reiser4_object_t *parent,
 			lost = !reiser4_key_compfull(&object->ent->parent,
 						     &sem->lost->ent->object);
 			
-			repair_semantic_lost_name(object, buff);
+			repair_semantic_lost_name(object, buff, 
+						  REISER4_MAX_BLKSIZE);
 			
-			if ((res = repair_semantic_unlink(sem, lost ? sem->lost
-							  : 0, object, buff)))
-				goto error_close_object;
+			res = lost ?
+				reiser4_object_unlink(sem->lost, buff):
+				reiser4_object_detach(object, NULL);
+			
+			if (res) goto error_close_object;
 		}
 
 		break;
@@ -441,8 +426,8 @@ static reiser4_object_t *cb_object_traverse(reiser4_object_t *parent,
 		aal_error("Semantic traverse failed to remove the "
 			  "entry \"%s\" [%s] pointing to [%s].", 
 			  entry->name, 
-			  reiser4_print_key(&entry->offset, PO_INODE),
-			  reiser4_print_key(&entry->object, PO_INODE));
+			  reiser4_print_inode(&entry->offset),
+			  reiser4_print_inode(&entry->object));
 	}
 	
  error_close_object:
@@ -574,14 +559,14 @@ static reiser4_object_t *repair_semantic_dir_open(repair_semantic_t *sem,
 
 		fsck_mess("The directory [%s] is recognized by the "
 			  "%s plugin which is not a directory one.", 
-			  reiser4_print_key(key, PO_INODE), 
+			  reiser4_print_inode(key), 
 			  object->ent->opset.plug[OPSET_OBJ]->label);
 		
 		reiser4_object_close(object);
 	} else {
 		/* No plugin was recognized. */
 		fsck_mess("Failed to recognize the plugin for the directory "
-			  "[%s].", reiser4_print_key(key, PO_INODE));
+			  "[%s].", reiser4_print_inode(key));
 	}
 	
 	if (sem->repair->mode != RM_BUILD)
@@ -589,7 +574,7 @@ static reiser4_object_t *repair_semantic_dir_open(repair_semantic_t *sem,
 
 	plug = reiser4_profile_plug(PROF_DIR);
 	fsck_mess("Trying to recover the directory [%s] with the default "
-		  "plugin--%s.", reiser4_print_key(key, PO_INODE), plug->label);
+		  "plugin--%s.", reiser4_print_inode(key), plug->label);
 
 	sem->stat.files++;
 	return repair_object_fake(tree, parent, key, plug);
@@ -628,14 +613,14 @@ static errno_t repair_semantic_object_check(repair_semantic_t *sem,
 			break;
 		
 		fsck_mess("Object [%s]: detaching.", 
-			 reiser4_print_key(&object->ent->object, PO_INODE));
+			 reiser4_print_inode(&object->ent->object));
 		
-		if ((res = repair_semantic_unlink(sem, NULL, object, NULL)))
+		if ((res = reiser4_object_detach(object, NULL)))
 			return res;
 		
 		fsck_mess("Object [%s]: attaching to [%s].", 
-			 reiser4_print_key(&object->ent->object, PO_INODE),
-			 reiser4_print_key(&object->ent->parent, PO_INODE));
+			 reiser4_print_inode(&object->ent->object),
+			 reiser4_print_inode(&object->ent->parent));
 
 		break;
 	}
@@ -702,7 +687,7 @@ static errno_t repair_semantic_lost_prepare(repair_semantic_t *sem) {
 		
 		fsck_mess("No 'lost+found' entry found. "
 			  "Building a new object with the key %s.",
-			  reiser4_print_key(&lost, PO_INODE));
+			  reiser4_print_inode(&lost));
 	} else {
 		reiser4_key_assign(&lost, &entry.object);
 	}
