@@ -3,7 +3,7 @@
    
    nodeptr40_repair.c -- repair default node pointer item plugin methods. */
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #include "nodeptr40.h"
 #include <repair/plugin.h>
 
@@ -57,7 +57,7 @@ void nodeptr40_print(reiser4_place_t *place, aal_stream_t *stream,
     
 	nodeptr = nodeptr40_body(place);
 
-	aal_stream_format(stream, "UNITS=1\n[%llu]\n", 
+	aal_stream_format(stream, "[%llu]\n", 
 			  np40_get_ptr(nodeptr));
 }
 #endif
