@@ -30,7 +30,7 @@ static errno_t key_large_assign(reiser4_key_t *dst,
 	aal_assert("umka-1111", src != NULL);
 
 	dst->plug = src->plug;
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	dst->adjust = src->adjust;
 #endif
 
@@ -40,7 +40,7 @@ static errno_t key_large_assign(reiser4_key_t *dst,
 	return 0;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Sets up key type */
 static void key_large_set_type(reiser4_key_t *key, 
 			       key_type_t type) 
@@ -166,7 +166,7 @@ static char *key_large_get_name(reiser4_key_t *key,
 	return name;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Sets up key offset */
 static void key_large_set_hash(reiser4_key_t *key, 
 			       uint64_t hash)
@@ -254,10 +254,10 @@ static int key_large_compfull(reiser4_key_t *key1,
 }
 
 /* Builds hash of the passed @name by means of using a hash plugin */
-static errno_t key_large_build_hash(reiser4_key_t *key,
-				    reiser4_plug_t *hash,
-				    reiser4_plug_t *fibre,
-				    char *name) 
+static void key_large_build_hash(reiser4_key_t *key,
+				 reiser4_plug_t *hash,
+				 reiser4_plug_t *fibre,
+				 char *name) 
 {
 	uint16_t len;
 	uint64_t offset;
@@ -265,7 +265,7 @@ static errno_t key_large_build_hash(reiser4_key_t *key,
 	uint64_t ordering;
     
 	if ((len = aal_strlen(name)) == 1 && name[0] == '.')
-		return 0;
+		return;
 
 	aal_assert("vpf-128", hash != NULL); 
 	aal_assert("vpf-1568", fibre != NULL);
@@ -297,18 +297,16 @@ static errno_t key_large_build_hash(reiser4_key_t *key,
 	key_large_set_ordering(key, ordering);
 	kl_set_objectid((key_large_t *)key->body, objectid);
 	key_large_set_offset(key, offset);
-
-	return 0;
 }
 
 /* Builds key by passed locality, objectid, and name. It is suitable for
    creating entry keys. */
-static errno_t key_large_build_hashed(reiser4_key_t *key,
-				      reiser4_plug_t *hash,
-				      reiser4_plug_t *fibre,
-				      uint64_t locality,
-				      uint64_t objectid,
-				      char *name) 
+static void key_large_build_hashed(reiser4_key_t *key,
+				   reiser4_plug_t *hash,
+				   reiser4_plug_t *fibre,
+				   uint64_t locality,
+				   uint64_t objectid,
+				   char *name) 
 {
 	key_type_t type;
 	
@@ -323,7 +321,7 @@ static errno_t key_large_build_hashed(reiser4_key_t *key,
 	kl_set_minor((key_large_t *)key->body,
 		      key_common_type2minor(type));
 	
-	return key_large_build_hash(key, hash, fibre, name);
+	key_large_build_hash(key, hash, fibre, name);
 }
 
 /* Builds generic key by all its components */
@@ -356,7 +354,7 @@ static errno_t key_large_build_generic(reiser4_key_t *key,
 	return 0;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 extern void key_large_print(reiser4_key_t *key,
 			    aal_stream_t *stream,
 			    uint16_t options);
@@ -377,7 +375,7 @@ static reiser4_key_ops_t key_large_ops = {
 	.build_hashed   = key_large_build_hashed,
 	.build_generic  = key_large_build_generic,
 	
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.check_struct	= key_large_check_struct,
 	.print		= key_large_print,
 
@@ -408,9 +406,9 @@ static reiser4_key_ops_t key_large_ops = {
 static reiser4_plug_t key_large_plug = {
 	.cl    = class_init,
 	.id    = {KEY_LARGE_ID, 0, KEY_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "key_large",
-	.desc  = "Large key for reiser4, ver. " VERSION,
+	.desc  = "Large key for reiser4. ",
 #endif
 	.o = {
 		.key_ops = &key_large_ops
