@@ -3,7 +3,7 @@
    
    dir40.c -- reiser4 directory object plugin. */
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #  include <unistd.h>
 #endif
 
@@ -25,7 +25,7 @@ static errno_t dir40_telldir(object_entity_t *entity,
 	plug_call(dir->position.plug->o.key_ops,
 		  assign, position, &dir->position);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Adjust is offset inside collided keys arrays and needed for
 	   positioning right in such a case. In normal case it is zero. */
 	position->adjust = dir->position.adjust;
@@ -56,7 +56,7 @@ static errno_t dir40_seekdir(object_entity_t *entity,
 	/* Getting adjust from the passed key and puting it to
 	   @dir->adjust. Seekdir is accepting key, which might be got from
 	   telldir() function. So, adjust will be set too. */
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	dir->position.adjust = position->adjust;
 #endif
 
@@ -75,7 +75,7 @@ errno_t dir40_reset(object_entity_t *entity) {
 	
 	/* Preparing key of the first entry in directory and set directory
 	   adjust to zero. */
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	dir->position.adjust = 0;
 #endif
 
@@ -94,10 +94,9 @@ errno_t dir40_reset(object_entity_t *entity) {
 errno_t dir40_fetch(dir40_t *dir, entry_hint_t *entry) {
 	trans_hint_t hint;
 
+	aal_memset(&hint, 0, sizeof(hint));
 	hint.count = 1;
 	hint.specific = entry;
-	hint.place_func = NULL;
-	hint.region_func = NULL;
 	hint.shift_flags = SF_DEFAULT;
 
 	/* Reading entry to passed @entry */
@@ -155,7 +154,7 @@ lookup_t dir40_next(dir40_t *dir, int first) {
 	return PRESENT;
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #define dir40_update_next(dir, first)			\
 {							\
 	/* Getting next directory item */		\
@@ -190,7 +189,7 @@ lookup_t dir40_update_body(object_entity_t *entity, int check_group) {
 	lookup_t res;
 	uint32_t units;
 	
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	uint32_t adjust = dir->position.adjust;
 #endif
 	
@@ -210,7 +209,7 @@ lookup_t dir40_update_body(object_entity_t *entity, int check_group) {
 		if (check_group && dir->body.plug->id.group != DIR_ITEM)
 			return ABSENT;
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 		/* No adjusting for the ABSENT result. */
 		adjust = 0;
 #endif
@@ -224,7 +223,7 @@ lookup_t dir40_update_body(object_entity_t *entity, int check_group) {
 	if (dir->body.pos.unit == MAX_UINT32)
 		dir->body.pos.unit = 0;
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	/* Adjusting current position by key's adjust. This is needed
 	   for working fine when key collisions take place. */
 	while (adjust || dir->body.pos.unit >= units) {
@@ -255,6 +254,26 @@ lookup_t dir40_update_body(object_entity_t *entity, int check_group) {
 	return PRESENT;
 }
 
+#ifndef ENABLE_MINIMAL
+static void dir40_entry_type(entry_hint_t *entry) {
+	entry->type = ET_NAME;
+
+	if (aal_strlen(entry->name) == 1 &&
+	    !aal_strncmp(entry->name, ".", 1))
+	{
+		entry->type = ET_SPCL;
+	}
+
+	if (aal_strlen(entry->name) == 2 &&
+	    !aal_strncmp(entry->name, "..",2))
+	{
+		entry->type = ET_SPCL;
+	}
+}
+#else
+#define dir40_entry_type(entity) do{;} while(0)
+#endif
+
 /* Reads one current directory entry to passed @entity hint. Returns count of
    read entries, zero for the case directory is over and nagtive values fopr
    errors. */
@@ -287,21 +306,7 @@ static int32_t dir40_readdir(object_entity_t *entity,
 	   completely -- and what is not -- that needs some other special
 	   actions, e.g. check_attach for ".." (and even "..." if it is needed
 	   one day), etc. */
-#ifndef ENABLE_STAND_ALONE
-	entry->type = ET_NAME;
-		
-	if (aal_strlen(entry->name) == 1 &&
-	    !aal_strncmp(entry->name, ".", 1))
-	{
-		entry->type = ET_SPCL;
-	}
-
-	if (aal_strlen(entry->name) == 2 &&
-	    !aal_strncmp(entry->name, "..",2))
-	{
-		entry->type = ET_SPCL;
-	}
-#endif
+	dir40_entry_type(entry);
 
 	units = plug_call(dir->body.plug->o.item_ops->balance,
 			  units, &dir->body);
@@ -322,7 +327,7 @@ static int32_t dir40_readdir(object_entity_t *entity,
 		if ((res = dir40_fetch(dir, &temp)))
 			return res;
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 		/* Taking care about adjust */
 		if (!plug_call(temp.offset.plug->o.key_ops,
 			       compfull, &temp.offset, &dir->position))
@@ -346,7 +351,7 @@ static lookup_t dir40_search(object_entity_t *entity, char *name,
 {
 	dir40_t *dir;
 	lookup_t res;
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	coll_hint_t hint;
 	coll_func_t func;
 #endif
@@ -365,14 +370,14 @@ static lookup_t dir40_search(object_entity_t *entity, char *name,
 		  obj40_locality(&dir->obj),
 		  obj40_objectid(&dir->obj), name);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	hint.specific = name;
 	hint.type = DIR_ITEM;
 	func = dir40_core->tree_ops.collision;
 #endif
 	
 	if ((res = obj40_find_item(&dir->obj, &dir->body.key, bias, 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 				   func, &hint,
 #else
 				   NULL, NULL,
@@ -394,6 +399,8 @@ static lookup_t dir40_search(object_entity_t *entity, char *name,
 		if (res == PRESENT) {
 			if (dir40_fetch(dir, entry))
 				return -EIO;
+
+			dir40_entry_type(entry);
 		}
 	}
 
@@ -445,7 +452,7 @@ static errno_t dir40_stat(object_entity_t *entity, stat_hint_t *hint) {
 	return obj40_load_stat(&dir->obj, hint);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Gets size from the object stat data */
 static uint64_t dir40_size(object_entity_t *entity) {
 	dir40_t *dir;
@@ -590,9 +597,9 @@ static errno_t dir40_truncate(object_entity_t *entity, uint64_t n) {
 		if (!obj40_belong(&place, dir->body.plug, &dir->position))
 			return 0;
 
+		aal_memset(&hint, 0, sizeof(hint));
+		
 		hint.count = 1;
-		hint.place_func = NULL;
-		hint.region_func = NULL;
 		hint.shift_flags = SF_DEFAULT;
 		place.pos.unit = MAX_UINT32;
 		
@@ -618,73 +625,14 @@ static errno_t dir40_clobber(object_entity_t *entity) {
 	return obj40_clobber(&((dir40_t *)entity)->obj);
 }
 
-/* Attaches passed directory denoted by @entity to @parent object. */
-static errno_t dir40_attach(object_entity_t *entity,
-			    object_entity_t *parent)
-{
-	errno_t res;
-	dir40_t *dir;
-	entry_hint_t entry;
-	
-	aal_assert("umka-2289", entity != NULL);
-	aal_assert("umka-2359", parent != NULL);
-
-	dir = (dir40_t *)entity;
-
-	aal_memset(&entry, 0, sizeof(entry));
-	
-	aal_strncpy(entry.name, "..", sizeof(entry.name));
-
-	/* Adding ".." pointing to parent to @entity object. */
-	plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops,
-		  assign, &entry.object, &parent->object);
-
-	if ((res = plug_call(entity->opset.plug[OPSET_OBJ]->o.object_ops,
-			     add_entry, entity, &entry)))
-	{
-		return res;
-	}
-
-	/* Increasing parent's @nlink by one */
-	return plug_call(parent->opset.plug[OPSET_OBJ]->o.object_ops, 
-			 link, parent);
-}
-
-/* Detaches @entity from @parent. */
-static errno_t dir40_detach(object_entity_t *entity,
-			    object_entity_t *parent)
-{
-	reiser4_plug_t *plug;
-	entry_hint_t entry;
-	dir40_t *dir;
-
-	aal_assert("umka-2291", entity != NULL);
-
-	dir = (dir40_t *)entity;
-
-	plug = entity->opset.plug[OPSET_OBJ];
-	
-	/* Removing ".." from child if it is found */
-	switch (plug_call(plug->o.object_ops, lookup, entity, "..", &entry)) {
-	case PRESENT:
-		return plug_call(plug->o.object_ops, rem_entry, entity, &entry);
-	default:
-		if (!parent) 
-			return 0;
-		
-		/* Decreasing parent's @nlink by one */
-		return plug_call(plug->o.object_ops, unlink, parent);
-	}
-}
-
 /* Return number of hard links. */
-static uint32_t dir40_links(object_entity_t *entity) {
+static bool_t dir40_linked(object_entity_t *entity) {
 	dir40_t *dir;
 	
 	aal_assert("umka-2294", entity != NULL);
 
 	dir = (dir40_t *)entity;
-	return obj40_links(&dir->obj);
+	return obj40_links(&dir->obj) != 1;
 }
 
 /* Addes one had link. */
@@ -723,11 +671,13 @@ static errno_t dir40_build_entry(object_entity_t *entity,
 	locality = obj40_locality(&dir->obj);
 	objectid = obj40_objectid(&dir->obj);
 	
-	return plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops, 
-			 build_hashed, &entry->offset, 
-			 entity->opset.plug[OPSET_HASH],
-			 entity->opset.plug[OPSET_FIBRE], 
-			 locality, objectid, entry->name);
+	plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops, 
+		  build_hashed, &entry->offset, 
+		  entity->opset.plug[OPSET_HASH],
+		  entity->opset.plug[OPSET_FIBRE], 
+		  locality, objectid, entry->name);
+
+	return 0;
 }
 
 /* Add new entry to directory. */
@@ -823,6 +773,8 @@ static errno_t dir40_rem_entry(object_entity_t *entity,
 	/* Looking for place to insert directory entry */
 	switch (dir40_search(entity, entry->name, FIND_EXACT, &temp)) {
 	case PRESENT:
+		aal_memset(&hint, 0, sizeof(hint));
+
 		hint.count = 1;
 		hint.place_func = NULL;
 		hint.region_func = NULL;
@@ -855,6 +807,78 @@ static errno_t dir40_rem_entry(object_entity_t *entity,
 	return obj40_touch(&dir->obj, size, bytes);
 }
 
+/* Attaches passed directory denoted by @entity to @parent object. */
+static errno_t dir40_attach(object_entity_t *entity,
+			    object_entity_t *parent)
+{
+	errno_t res;
+	dir40_t *dir;
+	entry_hint_t entry;
+	
+	aal_assert("umka-2289", entity != NULL);
+	aal_assert("umka-2359", parent != NULL);
+
+	dir = (dir40_t *)entity;
+
+	aal_memset(&entry, 0, sizeof(entry));
+	
+	aal_strncpy(entry.name, "..", sizeof(entry.name));
+
+	/* Adding ".." pointing to parent to @entity object. */
+	plug_call(STAT_KEY(&dir->obj)->plug->o.key_ops,
+		  assign, &entry.object, &parent->object);
+
+	if ((res = dir40_add_entry(entity, &entry)))
+		return res;
+
+	/* Increasing parent's @nlink by one */
+	return plug_call(parent->opset.plug[OPSET_OBJ]->o.object_ops, 
+			 link, parent);
+}
+
+/* Detaches @entity from @parent. */
+static errno_t dir40_detach(object_entity_t *entity,
+			    object_entity_t *parent)
+{
+	reiser4_plug_t *plug;
+	entry_hint_t entry;
+	uint32_t nlink;
+	dir40_t *dir;
+	errno_t res;
+
+	aal_assert("umka-2291", entity != NULL);
+
+	dir = (dir40_t *)entity;
+
+	if (parent) {
+		/* If @parent is given, check first that detaching is allowed
+		 -- i.e. nlink == 2. */
+
+		if ((res = obj40_update(&dir->obj)))
+			return res;
+		
+		if ((nlink = obj40_get_nlink(&dir->obj)) != 2) {
+			aal_error("Can't detach the object "
+				  "with nlink (%d).", nlink);
+			return -EINVAL;
+		}
+	}
+	
+	/* Removing ".." from child if it is found */
+	if (dir40_lookup(entity, "..", &entry) == PRESENT) {
+		if ((res = dir40_rem_entry(entity, &entry)))
+			return res;
+	}
+	
+	if (!parent) 
+		return 0;
+	
+	plug = parent->opset.plug[OPSET_OBJ];
+	
+	/* Decreasing parent's @nlink by one */
+	return plug_call(plug->o.object_ops, unlink, parent);
+}
+
 /* Directory enumerating related stuff.*/
 struct layout_hint {
 	void *data;
@@ -991,13 +1015,13 @@ static errno_t dir40_update(object_entity_t *entity,
 
 /* Directory object operations. */
 static reiser4_object_ops_t dir40_ops = {
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.create		= dir40_create,
 	.layout		= dir40_layout,
 	.metadata	= dir40_metadata,
 	.link		= dir40_link,
 	.unlink		= dir40_unlink,
-	.links		= dir40_links,
+	.linked		= dir40_linked,
 	.update         = dir40_update,
 	.truncate	= dir40_truncate,
 	.add_entry	= dir40_add_entry,
@@ -1032,9 +1056,9 @@ static reiser4_object_ops_t dir40_ops = {
 reiser4_plug_t dir40_plug = {
 	.cl    = class_init,
 	.id    = {OBJECT_DIR40_ID, DIR_OBJECT, OBJECT_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "dir40",
-	.desc  = "Compound directory for reiser4, ver. " VERSION,
+	.desc  = "Compound directory for reiser4. ",
 #endif
 	.o = {
 		.object_ops = &dir40_ops
