@@ -3,7 +3,7 @@
    
    reg40.c -- reiser4 regular file plugin. */
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 #  include <unistd.h>
 #endif
 
@@ -97,10 +97,9 @@ static int64_t reg40_read(object_entity_t *entity,
 	   initialize @count -- number of bytes to read, @specific -- pointer to
 	   buffer data will be read into, and pointer to tree instance, file is
 	   opened on. */ 
+	aal_memset(&hint, 0, sizeof(hint));
 	hint.count = n;
 	hint.specific = buff;
-	hint.place_func = NULL;
-	hint.region_func = NULL;
 
 	/* Initializing offset data must be read from. This is current file
 	   offset, so we use @reg->position. */
@@ -151,7 +150,7 @@ static object_entity_t *reg40_open(object_info_t *info) {
 	/* Reseting file (setting offset to 0) */
 	reg40_reset((object_entity_t *)reg);
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	{
 		lookup_t lookup;
 
@@ -181,7 +180,7 @@ static errno_t reg40_stat(object_entity_t *entity, stat_hint_t *hint) {
 	return obj40_load_stat(&reg->obj, hint);
 }
 
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 /* Create the file described by pased @hint. That is create files stat data
    item. */
 static object_entity_t *reg40_create(object_hint_t *hint) {
@@ -212,13 +211,13 @@ static object_entity_t *reg40_create(object_hint_t *hint) {
 
 /* Returns number of links. Needed to let higher API levels know, that file has
    zero links and may be clobbered. */
-static uint32_t reg40_links(object_entity_t *entity) {
+static bool_t reg40_linked(object_entity_t *entity) {
 	reg40_t *reg;
 	
 	aal_assert("umka-2296", entity != NULL);
 
 	reg = (reg40_t *)entity;
-	return obj40_links(&reg->obj);
+	return obj40_links(&reg->obj) != 0;
 }
 
 /* Increments link number in stat data. */
@@ -357,11 +356,10 @@ int64_t reg40_put(object_entity_t *entity, void *buff,
 	/* Preparing hint to be used for calling write method. This is
 	   initializing @count - number of bytes to write, @specific - buffer to
 	   write into and @offset -- file offset data must be written at. */
-	hint.bytes = 0;
+	aal_memset(&hint, 0, sizeof(hint));
 	hint.count = n;
 
 	hint.specific = buff;
-	hint.region_func = NULL;
 	hint.shift_flags = SF_DEFAULT;
 	hint.place_func = place_func;
 	hint.plug = reg->body_plug;
@@ -391,6 +389,8 @@ static int64_t reg40_cut(object_entity_t *entity, uint64_t offset) {
 
 	aal_assert("umka-3076", size > offset);
 
+	aal_memset(&hint, 0, sizeof(hint));
+	
 	/* Preparing key of the data to be truncated. */
 	plug_call(STAT_KEY(&reg->obj)->plug->o.key_ops,
 		  assign, &hint.offset, &reg->position);
@@ -399,8 +399,6 @@ static int64_t reg40_cut(object_entity_t *entity, uint64_t offset) {
 		  set_offset, &hint.offset, offset);
 
 	/* Removing data from the tree. */
-	hint.place_func = NULL;
-	hint.region_func = NULL;
 	hint.count = size - offset;
 	hint.shift_flags = SF_DEFAULT;
 	hint.data = reg->obj.info.tree;
@@ -464,13 +462,14 @@ static int64_t reg40_write(object_entity_t *entity,
 
 	/* Calculating new @bytes and updating stat data fields. */
 	bytes += obj40_get_bytes(&reg->obj);
-	return obj40_touch(&reg->obj, new_size, bytes);
+	if ((res = obj40_touch(&reg->obj, new_size, bytes)))
+		return res;
+	
+	return bytes;
 }
 
 /* Truncates file to passed size @n. */
-static errno_t reg40_truncate(object_entity_t *entity,
-			      uint64_t n)
-{
+static errno_t reg40_truncate(object_entity_t *entity, uint64_t n) {
 	errno_t res;
 	reg40_t *reg;
 	int64_t bytes;
@@ -502,7 +501,7 @@ static errno_t reg40_truncate(object_entity_t *entity,
 		return obj40_touch(&reg->obj, n, bytes);
 	} else {
 		/* Cutting items/units */
-		if ((bytes = reg40_cut(entity, size - n)) < 0)
+		if ((bytes = reg40_cut(entity, n)) < 0)
 			return bytes;
 
 		/* Updating stat data fields. */
@@ -684,7 +683,7 @@ static errno_t reg40_update(object_entity_t *entity, stat_hint_t *hint) {
 
 /* Regular file operations. */
 static reiser4_object_ops_t reg40_ops = {
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.create	        = reg40_create,
 	.write	        = reg40_write,
 	.truncate       = reg40_truncate,
@@ -694,7 +693,7 @@ static reiser4_object_ops_t reg40_ops = {
 	.update         = reg40_update,
 	.link           = reg40_link,
 	.unlink         = reg40_unlink,
-	.links          = reg40_links,
+	.linked         = reg40_linked,
 	.clobber        = reg40_clobber,
 	.recognize	= reg40_recognize,
 	.check_struct   = reg40_check_struct,
@@ -727,9 +726,9 @@ static reiser4_object_ops_t reg40_ops = {
 reiser4_plug_t reg40_plug = {
 	.cl    = class_init,
 	.id    = {OBJECT_REG40_ID, REG_OBJECT, OBJECT_PLUG_TYPE},
-#ifndef ENABLE_STAND_ALONE
+#ifndef ENABLE_MINIMAL
 	.label = "reg40",
-	.desc  = "Regular file for reiser4, ver. " VERSION,
+	.desc  = "Regular file for reiser4. ",
 #endif
 	.o = {
 		.object_ops = &reg40_ops
