@@ -35,10 +35,11 @@ static void cpfs_print_usage(char *name) {
     
 	fprintf(stderr, 
 		"Plugins options:\n"
-		"  -p, --print-params            prints default params.\n"
+		"  -p, --print-profile           prints the plugin profile.\n"
 		"  -l, --known-plugins           prints known plugins.\n"
 	        "  -o, --override TYPE=PLUGIN    overrides the default plugin of the type\n"
-	        "                                \"TYPE\" by the plugin \"PLUGIN\".\n"
+	        "                                \"TYPE\" by the plugin \"PLUGIN\" in the\n"
+		"                                profile.\n"
 		"Common options:\n"
 		"  -?, -h, --help                prints program usage.\n"
 		"  -V, --version                 prints current version.\n"
@@ -81,7 +82,7 @@ int main(int argc, char *argv[]) {
 		{"help", no_argument, NULL, 'h'},
 		{"force", no_argument, NULL, 'f'},
 		{"quiet", no_argument, NULL, 'q'},
-		{"print-params", no_argument, NULL, 'p'},
+		{"print-profile", no_argument, NULL, 'p'},
 		{"print-plugins", no_argument, NULL, 'l'},
 		{"override", required_argument, NULL, 'o'},
 		{"cache", required_argument, NULL, 'c'},
@@ -92,11 +93,12 @@ int main(int argc, char *argv[]) {
 	memset(override, 0, sizeof(override));
 
 	/* Parsing parameters */    
-	while ((c = getopt_long(argc, argv, "hVqfplo:c:", long_options, 
+	while ((c = getopt_long(argc, argv, "hVqfplo:c:?", long_options, 
 				(int *)0)) != EOF) 
 	{
 		switch (c) {
 		case 'h':
+		case '?':
 			cpfs_print_usage(argv[0]);
 			return NO_ERROR;
 		case 'V':
@@ -129,9 +131,6 @@ int main(int argc, char *argv[]) {
 			
 			aal_strncat(override, ",", 1);
 			break;
-		case '?':
-			cpfs_print_usage(argv[0]);
-			return NO_ERROR;
 		}
 	}
     
@@ -151,16 +150,16 @@ int main(int argc, char *argv[]) {
 		override[aal_strlen(override) - 1] = '\0';
 
 		if (!(flags & BF_QUIET)) {
-			aal_mess("Overriding default params "
-				 "by \"%s\".", override);
+			aal_mess("Overriding the plugin profile by \"%s\".",
+				 override);
 		}
 
-		if (misc_param_override(override))
+		if (misc_profile_override(override))
 			goto error_free_libreiser4;
 	}
 
 	if (flags & BF_SHOW_PARM)
-		misc_param_print();
+		misc_profile_print();
 	
 	if (flags & BF_SHOW_PLUG)
 		misc_plugins_print();
@@ -176,11 +175,6 @@ int main(int argc, char *argv[]) {
 		goto error_free_libreiser4;
 	}
 	
-	if (!(flags & BF_QUIET)) {
-		if (!(gauge = aal_gauge_create(GAUGE_SILENT, NULL)))
-			goto error_free_libreiser4;
-	}
-    
 	/* Checking is passed device is a block device. If so, we check also is
 	   it whole drive or just a partition. If the device is not a block
 	   device, then we emmit exception and propose user to use -f flag to
@@ -208,7 +202,7 @@ int main(int argc, char *argv[]) {
 	}
    
 	/* Checking if passed partition is mounted */
-	if (misc_dev_mounted(src_dev, "rw") && !(flags & BF_FORCE)) {
+	if (misc_dev_mounted(src_dev) == MF_RW && !(flags & BF_FORCE)) {
 		aal_error("Device %s is mounted for read/write "
 			  "at the moment. Use -f to force over.",
 			  src_dev);
@@ -239,7 +233,7 @@ int main(int argc, char *argv[]) {
 	}
    
 	/* Checking if passed partition is mounted */
-	if (misc_dev_mounted(dst_dev, NULL) && !(flags & BF_FORCE)) {
+	if (misc_dev_mounted(dst_dev) > 0 && !(flags & BF_FORCE)) {
 		aal_error("Device %s is mounted at the moment. "
 			  "Use -f to force over.", dst_dev);
 		goto error_free_libreiser4;
@@ -276,13 +270,15 @@ int main(int argc, char *argv[]) {
 		{
 			goto error_free_dst_device;
 		}
-	}
-    
-	if (gauge) {
-		aal_gauge_rename(gauge, "Copying %s to %s",
-				 src_dev, dst_dev);
 
-		aal_gauge_start(gauge);
+		if (!(gauge = aal_gauge_create(aux_gauge_handlers[GT_PROGRESS], 
+					       NULL, NULL, 0, "Copying %s to "
+					       "%s ... ", src_dev, dst_dev)))
+		{
+			goto error_free_libreiser4;
+		}
+		
+		aal_gauge_touch(gauge);
 	}
 
 	/* Opening source fs */
