@@ -392,11 +392,12 @@ int main(int argc, char *argv[]) {
 	}
     
 	/* Parsing parameters */    
-	while ((c = getopt_long(argc, argv, "hVqftb:djk:n:i:o:plsaPUOFWB:c:C",
+	while ((c = getopt_long(argc, argv, "hVqftb:djk:n:i:o:plsaPUOFWB:c:C?",
 				long_options, (int *)0)) != EOF) 
 	{
 		switch (c) {
 		case 'h':
+		case '?':
 			debugfs_print_usage(argv[0]);
 			return NO_ERROR;
 		case 'V':
@@ -471,9 +472,6 @@ int main(int argc, char *argv[]) {
 			aal_strncat(override, optarg, aal_strlen(optarg));
 			aal_strncat(override, ",", 1);
 			break;
-		case '?':
-			debugfs_print_usage(argv[0]);
-			return NO_ERROR;
 		case 'O':
 			space_flags = 0;
 			break;
@@ -698,27 +696,15 @@ int main(int argc, char *argv[]) {
 	}
 
 	if (behav_flags & BF_PACK_META) {
-//		void *error;
 		aal_stream_t stream;
 
 		aal_stream_init(&stream, stdout, &file_stream);
 		
-		/* FIXME-VITALY: This is needed to not print all found node
-		   corruptions, but this also avoid prining useful pack
-		   errors. */
-
-		/* FIXME-UMKA: Probably some kind of mask in libaal/exception
-		   for masking not needed exceptions would be helpful. */
-//		error = misc_exception_get_stream(EXCEPTION_TYPE_ERROR);
-//		misc_exception_set_stream(EXCEPTION_TYPE_ERROR, NULL);
-		
 		if (repair_fs_pack(fs, bitmap, &stream)) {
-//			misc_exception_set_stream(EXCEPTION_TYPE_ERROR, error);
 			aal_error("Can't pack filesystem.");
 			goto error_free_journal;
 		}
 		
-//		misc_exception_set_stream(EXCEPTION_TYPE_ERROR, error);
 		aal_stream_fini(&stream);
 		aux_bitmap_close(bitmap);
 		bitmap = NULL;
