@@ -603,7 +603,7 @@ static errno_t data_frag_process_node(reiser4_node_t *node, void *data) {
 		   the function we have seen abowe. */
 		if (reiser4_object_layout(object, file_frag_process_blk, data)) {
 			aal_error("Can't enumerate data blocks occupied by %s", 
-				  reiser4_print_key(&object->ent->object, PO_INODE));
+				  reiser4_print_inode(&object->ent->object));
 			goto error_close_object;
 		}
 
@@ -624,7 +624,7 @@ static errno_t data_frag_process_node(reiser4_node_t *node, void *data) {
 				(double)frag_hint->current / frag_hint->files : 0;
 			
 			aal_mess("Fragmentation for %s: %.6f [av. %.6f ]",
-				 reiser4_print_key(&object->ent->object, PO_INODE),
+				 reiser4_print_inode(&object->ent->object),
 				 file_factor, curr_factor);
 		}
 
@@ -653,9 +653,11 @@ errno_t measurefs_data_frag(reiser4_fs_t *fs, uint32_t flags) {
 			aal_gauge_create(aux_gauge_handlers[GT_PROGRESS],
 					 NULL, NULL, 0, "Data fragmentation "
 					 "... ");
-		if (!frag_hint.gauge)
+
+		if (!frag_hint.gauge) {
 			aal_fatal("Out of memory!");
 			return -ENOMEM;
+		}
 	}
 
 	frag_hint.flags = flags;
@@ -720,11 +722,12 @@ int main(int argc, char *argv[]) {
 	}
 
 	/* Parsing parameters */
-	while ((c = getopt_long(argc, argv, "hVqfKTDESF:o:plc:",
+	while ((c = getopt_long(argc, argv, "hVqfKTDESF:o:plc:?",
 				long_options, (int *)0)) != EOF)
 	{
 		switch (c) {
 		case 'h':
+		case '?':
 			measurefs_print_usage(argv[0]);
 			return NO_ERROR;
 		case 'V':
@@ -773,9 +776,6 @@ int main(int argc, char *argv[]) {
 
 			misc_mpressure_setup(cache);
 			break;
-		case '?':
-			measurefs_print_usage(argv[0]);
-			return NO_ERROR;
 		}
 	}
 
