@@ -91,12 +91,13 @@ static reiser4_object_t *reiser4_root_create(reiser4_fs_t *fs) {
 	hint.mode = 0;
 	reiser4_opset_profile(hint.info.opset.plug);
 	hint.info.opset.plug[OPSET_OBJ] = hint.info.opset.plug[OPSET_MKDIR];
+	hint.info.tree = (tree_entity_t *)fs->tree;
 
 	/* Preparing entry hint. */
 	entry.name[0] = '\0';
 	reiser4_key_assign(&entry.offset, &fs->tree->key);
 
-	return reiser4_object_create(fs->tree, &entry, &hint);
+	return reiser4_object_create(&entry, &hint);
 }
 
 int main(int argc, char *argv[]) {
@@ -151,11 +152,12 @@ int main(int argc, char *argv[]) {
 	memset(hint.label, 0, sizeof(hint.label));
 
 	/* Parsing parameters */    
-	while ((c = getopt_long(argc, argv, "hVqfb:U:L:splo:",
+	while ((c = getopt_long(argc, argv, "hVqfb:U:L:splo:?",
 				long_options, (int *)0)) != EOF) 
 	{
 		switch (c) {
 		case 'h':
+		case '?':
 			mkfs_print_usage(argv[0]);
 			return NO_ERROR;
 		case 'V':
@@ -217,9 +219,6 @@ int main(int argc, char *argv[]) {
 			aal_strncpy(hint.label, optarg,
 				    sizeof(hint.label));
 			break;
-		case '?':
-			mkfs_print_usage(argv[0]);
-			return NO_ERROR;
 		}
 	}
 
